<?php
/**
 * A TCP connection listener
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class TcpListener extends BaseListener implements Listenable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set the protocol to TCP
		$this->setProtocolName('tcp');
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$listenerInstance	An instance a prepared listener class
	 */
	public static final function createTcpListener () {
		// Get new instance
		$listenerInstance = new TcpListener();

		// Return the prepared instance
		return $listenerInstance;
	}

	/**
	 * Initializes the listener by setting up the required socket server
	 *
	 * @return	void
	 * @throws	InvalidSocketException	Thrown if the socket could not be initialized
	 */
	public function initListener () {
		// Create a streaming socket, of type TCP/IP
		$mainSocket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);

		// Is the socket resource valid?
		if (!is_resource($mainSocket)) {
			// Something bad happened
			throw new InvalidSocketException(array($this, $mainSocket), BaseListener::EXCEPTION_INVALID_SOCKET);
		} // END - if

		// Get socket error code for verification
		$socketError = socket_last_error($mainSocket);

		// Check if there was an error else
		if ($socketError > 0) {
			// Handle this socket error with a faked recipientData array
			$this->handleSocketError(__METHOD__, __LINE__, $mainSocket, array('0.0.0.0', '0'));
			/*
			// Then throw again
			throw new InvalidSocketException(array($this, $mainSocket, $socketError, socket_strerror($socketError)), BaseListener::EXCEPTION_INVALID_SOCKET);
			*/
		} // END - if

		// Set the option to reuse the port
		if (!socket_set_option($mainSocket, SOL_SOCKET, SO_REUSEADDR, 1)) {
			// Handle this socket error with a faked recipientData array
			$this->handleSocketError(__METHOD__, __LINE__, $mainSocket, array('0.0.0.0', '0'));
			/*
			// Get socket error code for verification
			$socketError = socket_last_error($mainSocket);

			// Get error message
			$errorMessage = socket_strerror($socketError);

			// Shutdown this socket
			$this->shutdownSocket($mainSocket);

			// And throw again
			throw new InvalidSocketException(array($this, $mainSocket, $socketError, $errorMessage), BaseListener::EXCEPTION_INVALID_SOCKET);
			*/
		} // END - if

		/*
		 * "Bind" the socket to the given address, on given port so this means
		 * that all connections on this port are now our resposibility to
		 * send/recv data, disconnect, etc..
		 */
		self::createDebugInstance(__CLASS__)->debugOutput('TCP-LISTENER[' . __METHOD__ . ':' . __LINE__ . ']: Binding to address ' . $this->getListenAddress() . ':' . $this->getListenPort());
		if (!socket_bind($mainSocket, $this->getListenAddress(), $this->getListenPort())) {
			// Handle this socket error with a faked recipientData array
			$this->handleSocketError(__METHOD__, __LINE__, $mainSocket, array('0.0.0.0', '0'));
			/*
			// Get socket error code for verification
			$socketError = socket_last_error($mainSocket);

			// Get error message
			$errorMessage = socket_strerror($socketError);

			// Shutdown this socket
			$this->shutdownSocket($mainSocket);

			// And throw again
			throw new InvalidSocketException(array($this, $mainSocket, $socketError, $errorMessage), BaseListener::EXCEPTION_INVALID_SOCKET);
			*/
		} // END - if

		// Start listen for connections
		self::createDebugInstance(__CLASS__)->debugOutput('TCP-LISTENER[' . __METHOD__ . ':' . __LINE__ . ']: Listening for connections.');
		if (!socket_listen($mainSocket)) {
			// Handle this socket error with a faked recipientData array
			$this->handleSocketError(__METHOD__, __LINE__, $mainSocket, array('0.0.0.0', '0'));
			/*
			// Get socket error code for verification
			$socketError = socket_last_error($mainSocket);

			// Get error message
			$errorMessage = socket_strerror($socketError);

			// Shutdown this socket
			$this->shutdownSocket($mainSocket);

			// And throw again
			throw new InvalidSocketException(array($this, $mainSocket, $socketError, $errorMessage), BaseListener::EXCEPTION_INVALID_SOCKET);
			*/
		} // END - if

		// Now, we want non-blocking mode
		self::createDebugInstance(__CLASS__)->debugOutput('TCP-LISTENER[' . __METHOD__ . ':' . __LINE__ . ']: Setting non-blocking mode.');
		if (!socket_set_nonblock($mainSocket)) {
			// Handle this socket error with a faked recipientData array
			$this->handleSocketError(__METHOD__, __LINE__, $mainSocket, array('0.0.0.0', '0'));
			/*
			// Get socket error code for verification
			$socketError = socket_last_error($mainSocket);

			// Get error message
			$errorMessage = socket_strerror($socketError);

			// Shutdown this socket
			$this->shutdownSocket($mainSocket);

			// And throw again
			throw new InvalidSocketException(array($this, $mainSocket, $socketError, $errorMessage), BaseListener::EXCEPTION_INVALID_SOCKET);
			*/
		} // END - if

		// Set the main socket
		$this->registerServerSocketResource($mainSocket);

		// Initialize the peer pool instance
		$poolInstance = ObjectFactory::createObjectByConfiguredName('node_pool_class', array($this));

		// Add main socket
		$poolInstance->addPeer($mainSocket, BaseConnectionHelper::CONNECTION_TYPE_SERVER);

		// And add it to this listener
		$this->setPoolInstance($poolInstance);

		// Initialize iterator for listening on packages
		$iteratorInstance = ObjectFactory::createObjectByConfiguredName('network_listen_iterator_class', array($poolInstance->getPoolEntriesInstance()));

		// Rewind it and remember it in this class
		$iteratorInstance->rewind();
		$this->setIteratorInstance($iteratorInstance);

		// Initialize the network package handler
		$handlerInstance = ObjectFactory::createObjectByConfiguredName('tcp_raw_data_handler_class');

		// Set it in this class
		$this->setHandlerInstance($handlerInstance);

		// Output message
		self::createDebugInstance(__CLASS__)->debugOutput('TCP-LISTENER[' . __METHOD__ . ':' . __LINE__ . ']: TCP listener now ready on IP ' . $this->getListenAddress() . ', port ' . $this->getListenPort() . ' for service.');
	}

	/**
	 * "Listens" for incoming network packages
	 *
	 * @return	void
	 */
	public function doListen () {
		// Call super method
		$this->doListenSocketSelect(':0');
	}

	/**
	 * Checks whether the listener would accept the given package data array
	 *
	 * @param	$packageData	Raw package data
	 * @return	$accepts		Whether this listener does accept
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function ifListenerAcceptsPackageData (array $packageData) {
		// Please don't call this
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}
}

// [EOF]
?>
